@@ -1,5 +1,4 @@
 # Functitions for specifying constraints in the constrained approach
-# modsem(method = "ca"). Last updated: 29.05.2024
 
 
 labelFactorLoadings <- function(parTable) {
@@ -14,14 +13,17 @@ labelFactorLoadings <- function(parTable) {
 
 specifyFactorLoadingsSingle <- function(parTable, relDf) {
   latentProdName <- stringr::str_c(rownames(relDf), collapse = "")
+
   for (indProd in colnames(relDf)) {
-    indProdLabel <- createLabelLambda(indProd, latentProdName)
+    indProdLabel     <- createLabelLambda(indProd, latentProdName)
     indsInProdLabels <- createLabelLambda(relDf[[indProd]], rownames(relDf))
-    vecLhsRhs <- c(indProdLabel, stringr::str_c(indsInProdLabels,
-                                                collapse = " * "))
-    newRow <- createParTableRow(vecLhsRhs, op = "==")
+    vecLhsRhs        <- c(indProdLabel, stringr::str_c(indsInProdLabels,
+                                                       collapse = " * "))
+
+    newRow   <- createParTableRow(vecLhsRhs, op = "==")
     parTable <- rbind(parTable, newRow)
   }
+
   parTable
 }
 
@@ -34,28 +36,28 @@ specifyFactorLoadings <- function(parTable, relDfs) {
 
 
 addVariances <- function(pt) {
-  # Add variance-labels if missing for latents
-  latents <- unique(pt[pt$op == "=~", "lhs"])
-  if (length(latents) == 0) return(pt)
+  # Add variance-labels if missing for LVs
+  LVs <- getLVs(pt)
+  if (length(LVs) == 0) return(pt)
 
-  specifiedLvs <- pt[pt$lhs %in% latents &
+  specifiedLVs <- pt[pt$lhs %in% LVs &
                      pt$op == "~~" &
-                     pt$rhs %in% latents &
+                     pt$rhs %in% LVs &
                      pt$lhs == pt$rhs, "lhs"] |> unique()
-  toBeSpecifiedLvs <- latents[!latents %in% specifiedLvs]
+  toBeSpecifiedLvs <- LVs[!LVs %in% specifiedLVs]
 
-  newRows <- lapply(toBeSpecifiedLvs, FUN = function(x) 
+  newRows <- lapply(toBeSpecifiedLvs, FUN = function(x)
                     createParTableRow(c(x, x), op = "~~")) |>
     purrr::list_rbind()
   pt <- rbind(pt, newRows)
 
-  # Add variance for observed variables if missing 
-  observed <- unique(pt[pt$op == "=~", "rhs"])
+  # Add variance for observed variables if missing
+  observed <- getIndicators(pt, observed = TRUE)
 
   specifiedOvs <- pt[pt$lhs %in% observed &
-                        pt$op == "~~" &
-                        pt$rhs %in% observed &
-                        pt$lhs == pt$rhs, "lhs"] |> unique()
+                     pt$op == "~~" &
+                     pt$rhs %in% observed &
+                     pt$lhs == pt$rhs, "lhs"] |> unique()
   toBeSpecifiedOvs <- observed[!observed %in% specifiedOvs]
 
   newRows <- lapply(toBeSpecifiedOvs,
@@ -67,36 +69,60 @@ addVariances <- function(pt) {
 
 addCovariances <- function(pt) {
   # Add covariances for exogenous variables if missing
-  latents <- unique(pt[pt$op == "=~", "lhs"])
+  pt       <- stripColonsParTable(pt)
+  latents  <- getLVs(pt)
   if (length(latents) == 0) return(pt)
 
-  combos <- getUniqueCombos(latents)
-  combos$connected <- !is.na(apply(combos, MARGIN = 1, function(xy)
-                                   trace_path(pt, xy[[1]], xy[[2]])))
-  toBeSpecified <- combos[!combos$connected, c("V1", "V2")]
-  newRows <- apply(toBeSpecified[c("V1", "V2")],
-                   MARGIN = 1,
-                   FUN = function(x) createParTableRow(x, op = "~~")) |>
-    purrr::list_rbind()
-  rbind(pt, newRows)
+  higherOrder <- getHigherOrderLVs(pt)
+  firstOrder  <- latents[!latents %in% higherOrder]
+
+  if (length(higherOrder)) { # then we should not touch the first order covariances
+    combosHO <- getUniqueCombos(higherOrder)
+    ptHO     <- pt[pt$lhs %in% higherOrder & pt$rhs %in% higherOrder, ]
+    combosHO$connected <- !is.na(apply(combosHO, MARGIN = 1, function(xy)
+                                       trace_path(ptHO, xy[[1]], xy[[2]])))
+    toBeSpecified <- combosHO[!combosHO$connected, c("V1", "V2")]
+    if (nrow(toBeSpecified)) {
+      newRows <- apply(toBeSpecified[c("V1", "V2")],
+                       MARGIN = 1,
+                       FUN = function(x) createParTableRow(x, op = "~~")) |>
+        purrr::list_rbind()
+      pt <- rbind(pt, newRows)
+    }
+  } else {
+    combosFO <- getUniqueCombos(firstOrder)
+    ptFO     <- pt[pt$lhs %in% firstOrder & pt$rhs %in% firstOrder, ]
+    combosFO$connected <- !is.na(apply(combosFO, MARGIN = 1, function(xy)
+                                       trace_path(ptFO, xy[[1]], xy[[2]])))
+    toBeSpecified <- combosFO[!combosFO$connected, c("V1", "V2")]
+    if (nrow(toBeSpecified)) {
+      newRows <- apply(toBeSpecified[c("V1", "V2")],
+                       MARGIN = 1,
+                       FUN = function(x) createParTableRow(x, op = "~~")) |>
+        purrr::list_rbind()
+      pt <- rbind(pt, newRows)
+    }
+  }
+
+  pt
 }
 
 
 labelParameters <- function(pt) {
-  latents <- unique(pt[pt$op == "=~", "lhs"]) 
+  latents <- getLVs(pt)
   endogenous <- latents[latents %in% pt[pt$op == "~", "lhs"]]
   exogenous <- latents[!latents %in% endogenous]
-  observed <- unique(pt[pt$op == "=~", "rhs"])
+  observed <- getOVs(pt)
 
   # Gamma
   pt[pt$op == "~", "mod"] <-
     apply(pt[pt$op == "~", c("rhs", "lhs")],
-          MARGIN = 1, FUN = function(x) 
+          MARGIN = 1, FUN = function(x)
             createLabelGamma(x[[1]], x[[2]]))
 
   # Variances of exogenous
   pt[pt$op == "~~" & pt$lhs == pt$rhs & pt$lhs %in% exogenous, "mod"] <-
-    vapply(pt[pt$op == "~~" & pt$lhs == pt$rhs & 
+    vapply(pt[pt$op == "~~" & pt$lhs == pt$rhs &
               pt$lhs %in% exogenous, "lhs"],
            FUN.VALUE = vector("character", length = 1L),
            FUN = createLabelVar)
@@ -108,7 +134,7 @@ labelParameters <- function(pt) {
            FUN.VALUE = vector("character", length = 1L),
            FUN = createLabelZeta)
 
-  # Variance of Observed Variables  
+  # Variance of Observed Variables
   pt[pt$op == "~~" & pt$rhs %in% observed & pt$lhs == pt$rhs, "mod"] <-
     vapply(pt[pt$op == "~~" & pt$rhs %in% observed &
            pt$lhs == pt$rhs, "rhs"],
@@ -118,7 +144,7 @@ labelParameters <- function(pt) {
   # Covariances
   pt[pt$op == "~~" & pt$lhs != pt$rhs, "mod"] <-
     apply(pt[pt$op == "~~" & pt$lhs != pt$rhs, c("lhs", "rhs")],
-          MARGIN = 1, FUN = function(x) 
+          MARGIN = 1, FUN = function(x)
             createLabelCov(x[[1]], x[[2]]))
 
   pt
@@ -129,39 +155,40 @@ specifyVarCovSingle <- function(parTable, relDf) {
   # This function specifies variances for latents, indicators,
   # and indicator products. It will also specifies covariances for latent
   # products, and elements int those products.
-  if (nrow(relDf) > 2) {
-    stop2("Constraints for products with more than two ",
+  stopif(nrow(relDf) > 2, "Constraints for products with more than two ",
          " elements are not supported for this method")
-  }
+
   # General info
   elemsInProdTerm <- rownames(relDf)
   latentProd <- stringr::str_c(rownames(relDf), collapse = "")
 
   # Variance of latent product
   labelLatentProd <- createLabelVar(latentProd)
-  labelsElemsInProd <- vapply(elemsInProdTerm, 
+  labelsElemsInProd <- vapply(elemsInProdTerm,
                               FUN.VALUE = vector("character", length = 1L),
                               FUN = function(x) trace_path(parTable, x, x))
 
   labelCovElems <- trace_path(parTable, elemsInProdTerm[[1]],
-                             elemsInProdTerm[[2]]) |> paste0(" ^ 2")
+                              elemsInProdTerm[[2]]) |> paste0(" ^ 2")
 
   lhs <- labelLatentProd
   rhs <- paste(stringr::str_c(labelsElemsInProd, collapse = " * "),
-               labelCovElems,
-               sep = " + ")
+               labelCovElems, sep = " + ")
   varLatentProd <- createParTableRow(c(lhs, rhs), op = "==")
 
   # covariances between elems and latents
   labelsCovElemProd <-
-    vapply(elemsInProdTerm,
-      FUN.VALUE = vector("character", length = 1L),
-      FUN = function(elem) createLabelCov(elem, latentProd)) # wrap in anonymous fun
+    vapply(elemsInProdTerm, FUN.VALUE = vector("character", length = 1L),
+           FUN = function(elem) createLabelCov(elem, latentProd)) # wrap in anonymous fun
                                                              # scope latentProd
+  
+  labelsCovElemProd <- labelsCovElemProd[labelsCovElemProd %in% parTable$mod]
 
-  covsElemsProd <- lapply(labelsCovElemProd, FUN = function(x) 
-                            createParTableRow(c(x, "0"), op = "==")) |>
-    purrr::list_rbind()
+  if (length(labelsCovElemProd)) { # should not be added in higher order models
+    covsElemsProd <- lapply(labelsCovElemProd, FUN = function(x)
+                              createParTableRow(c(x, "0"), op = "==")) |>
+      purrr::list_rbind()
+  } else covsElemsProd <- NULL
 
   # Variances of product indicators
   constrained.varProdInds <- vector("list", length = ncol(relDf))
@@ -170,30 +197,26 @@ specifyVarCovSingle <- function(parTable, relDf) {
     labelVarIndProd <- createLabelVar(indProd)
 
     labelsFactorLoadings <- vector("character", length = nrow(relDf))
-    labelsVarLatents <- vector("character", length = nrow(relDf))
-    labelsVarInds <- vector("character", length = nrow(relDf))
+    labelsVarLatents     <- vector("character", length = nrow(relDf))
+    labelsVarInds        <- vector("character", length = nrow(relDf))
 
-    for (latent in 1:nrow(relDf)) {
-      labelsFactorLoadings[[latent]] <- 
-        createLabelLambdaSquared(relDf[latent, indProd], 
+    for (latent in seq_len(nrow(relDf))) {
+      labelsFactorLoadings[[latent]] <-
+        createLabelLambdaSquared(relDf[latent, indProd],
                                  rownames(relDf)[[latent]])
-      labelsVarLatents[[latent]] <- 
+      labelsVarLatents[[latent]] <-
         trace_path(parTable, rownames(relDf)[[latent]],
-                  rownames(relDf)[[latent]])
+                   rownames(relDf)[[latent]])
       labelsVarInds[[latent]] <- createLabelVar(relDf[latent, indProd])
     }
 
-    lhs <- labelVarIndProd
-    rhs1 <- paste(labelsFactorLoadings[[1]],
-                  labelsVarLatents[[1]],
-                  labelsVarInds[[2]],
-                  sep = " * ")
-    rhs2 <- paste(labelsFactorLoadings[[2]],
-                  labelsVarLatents[[2]],
-                  labelsVarInds[[1]],
-                  sep = " * ")
+    lhs  <- labelVarIndProd
+    rhs1 <- paste(labelsFactorLoadings[[1]], labelsVarLatents[[1]],
+                  labelsVarInds[[2]], sep = " * ")
+    rhs2 <- paste(labelsFactorLoadings[[2]], labelsVarLatents[[2]],
+                  labelsVarInds[[1]], sep = " * ")
     rhs3 <- paste(labelsVarInds[[1]], labelsVarInds[[2]], sep = " * ")
-    rhs <- paste(rhs1, rhs2, rhs3, sep = " + ")
+    rhs  <- paste(rhs1, rhs2, rhs3, sep = " + ")
 
     constrained.varProdInds[[indProd]] <- createParTableRow(c(lhs, rhs), op = "==")
   }
